@@ -160,7 +160,7 @@ static void fp_set(element_ptr c, element_ptr a) {
     mpz_t z1, z2;
     z1->_mp_d = cd->d;
     z2->_mp_d = ad->d;
-    z1->_mp_size = z1->_mp_alloc = z2->_mp_size = z2->_mp_alloc = p->limbs;
+    z1->_mp_size = z1->_mp_alloc = z2->_mp_size = z2->_mp_alloc = (int)p->limbs;
     mpz_set(z1, z2);
 
     cd->flag = 2;
@@ -232,7 +232,7 @@ static void fp_halve(element_ptr c, element_ptr a) {
   } else {
     fp_field_data_ptr p = c->field->data;
     const size_t t = p->limbs;
-    int carry = 0;
+    mp_limb_t carry = 0;
     mp_limb_t *alimb = ad->d;
     mp_limb_t *climb = cd->d;
     if (alimb[0] & 1) {
@@ -290,8 +290,9 @@ static void fp_mul(element_ptr c, element_ptr a, element_ptr b) {
     size_t t = p->limbs;
     //mp_limb_t tmp[3 * t + 1];
     //mp_limb_t *qp = &tmp[2 * t];
-    mp_limb_t tmp[2 * t];
-    mp_limb_t qp[t + 1];
+    mp_limb_t* buf = pbc_malloc(((2 * t) + (t + 1)) * sizeof(mp_limb_t));
+    mp_limb_t* tmp = buf;
+    mp_limb_t* qp = &buf[2 * t];
     //static mp_limb_t tmp[2 * 100];
     //static mp_limb_t qp[100 + 1];
 
@@ -299,6 +300,8 @@ static void fp_mul(element_ptr c, element_ptr a, element_ptr b) {
 
     mpn_tdiv_qr(qp, cd->d, 0, tmp, 2 * t, p->primelimbs, t);
     cd->flag = 2;
+
+    pbc_free(buf);
   }
 }
 
@@ -314,12 +317,12 @@ static void fp_square(element_ptr c, element_ptr a) {
   } else {
     cd->flag = 2;
     z1->_mp_d = cd->d;
-    z1->_mp_size = z1->_mp_alloc = p->limbs;
+    z1->_mp_size = z1->_mp_alloc = (int)p->limbs;
     if (c == a) {
       mpz_powm_ui(z1, z1, 2, c->field->order);
     } else {
       z2->_mp_d = ad->d;
-      z2->_mp_size = z2->_mp_alloc = p->limbs;
+      z2->_mp_size = z2->_mp_alloc = (int)p->limbs;
       mpz_powm_ui(z1, z2, 2, c->field->order);
     }
 
@@ -351,11 +354,12 @@ static void fp_mul_si(element_ptr c, element_ptr a, signed long int op) {
     cd->flag = 2;
     fp_field_data_ptr p = a->field->data;
     size_t t = p->limbs;
-    mp_limb_t tmp[t + 1];
+    mp_limb_t* tmp = pbc_malloc((t + 1) * sizeof(mp_limb_t));
     mp_limb_t qp[2];
 
     tmp[t] = mpn_mul_1(tmp, ad->d, t, labs(op));
     mpn_tdiv_qr(qp, cd->d, 0, tmp, t + 1, p->primelimbs, t);
+    pbc_free(tmp);
     if (op < 0) {               //TODO: don't need to check c != 0 this time
       fp_neg(c, c);
     }
@@ -403,7 +407,7 @@ static void fp_random(element_ptr a) {
   mpz_clear(z);
 }
 
-static void fp_from_hash(element_ptr a, void *data, int len) {
+static void fp_from_hash(element_ptr a, const void *data, int len) {
   mpz_t z;
 
   mpz_init(z);
@@ -433,11 +437,14 @@ static int fp_sgn_even(element_ptr a) {
   fp_field_data_ptr p = a->field->data;
   dataptr ad = a->data;
   if (!ad->flag) return 0;
-  mp_limb_t sum[p->limbs];
+  mp_limb_t* sum = pbc_malloc(p->limbs * sizeof(mp_limb_t));
 
-  int carry = mpn_add_n(sum, ad->d, ad->d, p->limbs);
-  if (carry) return 1;
-  return mpn_cmp(sum, p->primelimbs, p->limbs);
+  mp_limb_t carry = mpn_add_n(sum, ad->d, ad->d, p->limbs);
+  int result;
+  if (carry) result = 1;
+  else result = mpn_cmp(sum, p->primelimbs, p->limbs);
+  pbc_free(sum);
+  return result;
 }
 
 
@@ -470,7 +477,7 @@ static int fp_to_bytes(unsigned char *data, element_t a) {
   return n;
 }
 
-static int fp_from_bytes(element_t a, unsigned char *data) {
+static int fp_from_bytes(element_t a, const unsigned char *data) {
   dataptr ad = a->data;
   int n;
   mpz_t z;
@@ -542,5 +549,5 @@ void field_init_faster_fp(field_ptr f, mpz_t prime) {
   mpz_export(p->primelimbs, &p->limbs, -1, sizeof(mp_limb_t), 0, 0, prime);
 
   mpz_set(f->order, prime);
-  f->fixed_length_in_bytes = (mpz_sizeinbase(prime, 2) + 7) / 8;
+  f->fixed_length_in_bytes = (int)((mpz_sizeinbase(prime, 2) + 7) / 8);
 }
