@@ -363,7 +363,7 @@ static void f_random(element_ptr n) {
   multiz_free(delme);
 }
 
-static void f_from_hash(element_ptr n, void *data, int len) {
+static void f_from_hash(element_ptr n, const void *data, int len) {
   mpz_t z;
   mpz_init(z);
   mpz_import(z, len, -1, 1, -1, 0, data);
@@ -470,14 +470,14 @@ static int z_to_bytes(unsigned char *data, element_t e) {
   }
   n += (msb + 7) / 8 - 4;
   for (i=0; i<4; i++) {
-    data[i] = (n >> 8 * (3 - i));
+    data[i] = (unsigned char)(n >> 8 * (3 - i));
   }
   n += 4;
 
-  return n;
+  return (int)n;
 }
 
-static int z_from_bytes(element_t e, unsigned char *data) {
+static int z_from_bytes(element_t e, const unsigned char *data) {
   unsigned char *ptr;
   size_t i, n;
   mpz_ptr z = e->data;
@@ -495,7 +495,7 @@ static int z_from_bytes(element_t e, unsigned char *data) {
   }
   if (data[4] & 128) {
     neg = 1;
-    data[4] &= 127;
+    ptr[4] &= 127;
   }
   for (i=0; i<n; i++) {
     mpz_set_ui(z1, *ptr);
@@ -505,11 +505,11 @@ static int z_from_bytes(element_t e, unsigned char *data) {
   }
   mpz_clear(z1);
   if (neg) mpz_neg(z, z);
-  return n;
+  return (int)n;
 }
 
 static int z_length_in_bytes(element_ptr a) {
-  return (mpz_sizeinbase(a->data, 2) + 7) / 8 + 4;
+  return (int)((mpz_sizeinbase(a->data, 2) + 7) / 8 + 4);
 }
 
 static void f_out_info(FILE *out, field_ptr f) {
