@@ -338,7 +338,7 @@ static void polymod_random(element_ptr e) {
   }
 }
 
-static void polymod_from_hash(element_ptr e, void *data, int len) {
+static void polymod_from_hash(element_ptr e, const void *data, int len) {
   // TODO: Improve this.
   element_t *coeff = e->data;
   int i, n = polymod_field_degree(e->field);
@@ -399,7 +399,7 @@ static int poly_snprint(char *s, size_t size, element_ptr e) {
   }
   status = snprintf(s + result, left, "]");
   if (status < 0) return status;
-  return result + status;
+  return (int)(result + status);
   #undef clip_sub
 }
 
@@ -575,7 +575,7 @@ static int poly_to_bytes(unsigned char *buf, element_t p) {
   return result;
 }
 
-static int poly_from_bytes(element_t p, unsigned char *buf) {
+static int poly_from_bytes(element_t p, const unsigned char *buf) {
   int result = 2;
   int count = buf[0] + buf[1] * 256;
   int i;
@@ -671,7 +671,7 @@ static void polymod_sqrt(element_ptr res, element_ptr a) {
     element_set1(s);
     //TODO: this can be optimized greatly
     //since we know r has the form ax + b
-    for (i = mpz_sizeinbase(z, 2) - 1; i >=0; i--) {
+    for (i = (int)mpz_sizeinbase(z, 2) - 1; i >=0; i--) {
       element_mul(s, s, s);
       if (poly_degree(s) == 2) {
         e1 = poly_coeff(s, 0);
@@ -739,7 +739,7 @@ static int polymod_length_in_bytes(element_t f) {
   return res;
 }
 
-static int polymod_from_bytes(element_t f, unsigned char *data) {
+static int polymod_from_bytes(element_t f, const unsigned char *data) {
   mfptr p = f->field->data;
   element_t *coeff = f->data;
   int i, n = p->n;
@@ -1225,7 +1225,7 @@ static int polymod_snprint(char *s, size_t size, element_ptr e) {
   size_t result = 0, left;
   int status;
 
-  #define clip_sub(void) {                     \
+  #define clip_sub() {                         \
     result += status;                          \
     left = result >= size ? 0 : size - result; \
   }
@@ -1246,7 +1246,7 @@ static int polymod_snprint(char *s, size_t size, element_ptr e) {
   }
   status = snprintf(s + result, left, "]");
   if (status < 0) return status;
-  return result + status;
+  return (int)(result + status);
   #undef clip_sub
 }
 
@@ -1280,7 +1280,7 @@ static int polymod_set_str(element_ptr e, const char *s, int base) {
     if (i<n-1 && *cp++ != ',') return 0;
   }
   if (*cp++ != ']') return 0;
-  return cp - s;
+  return (int)(cp - s);
 }
 
 static int polymod_coeff_count(element_ptr e) {
