@@ -107,7 +107,7 @@ static void z_random(element_ptr n) {
   mpz_set_ui(n->data, 0);
 }
 
-static void z_from_hash(element_ptr n, void *data, int len) {
+static void z_from_hash(element_ptr n, const void *data, int len) {
   mpz_import(n->data, len, -1, 1, -1, 0, data);
 }
 
@@ -158,14 +158,14 @@ static int z_to_bytes(unsigned char *data, element_t e) {
   }
   n += (msb + 7) / 8 - 4;
   for (i=0; i<4; i++) {
-    data[i] = (n >> 8 * (3 - i));
+    data[i] = (unsigned char)(n >> 8 * (3 - i));
   }
   n += 4;
 
-  return n;
+  return (int)n;
 }
 
-static int z_from_bytes(element_t e, unsigned char *data) {
+static int z_from_bytes(element_t e, const unsigned char *data) {
   unsigned char *ptr;
   size_t i, n;
   mpz_ptr z = e->data;
@@ -183,7 +183,7 @@ static int z_from_bytes(element_t e, unsigned char *data) {
   }
   if (data[4] & 128) {
     neg = 1;
-    data[4] &= 127;
+    ptr[4] &= 127;
   }
   for (i=0; i<n; i++) {
     mpz_set_ui(z1, *ptr);
@@ -193,7 +193,7 @@ static int z_from_bytes(element_t e, unsigned char *data) {
   }
   mpz_clear(z1);
   if (neg) mpz_neg(z, z);
-  return n;
+  return (int)n;
 }
 
 static void z_to_mpz(mpz_ptr z, element_ptr a) {
@@ -201,7 +201,7 @@ static void z_to_mpz(mpz_ptr z, element_ptr a) {
 }
 
 static int z_length_in_bytes(element_ptr a) {
-  return (mpz_sizeinbase(a->data, 2) + 7) / 8 + 4;
+  return (int)((mpz_sizeinbase(a->data, 2) + 7) / 8 + 4);
 }
 
 static void z_out_info(FILE *out, field_ptr f) {
