@@ -103,7 +103,7 @@ static inline void d_miller_evalfn(element_t e0,
 
 static void cc_miller_no_denom_proj(element_t res, mpz_t q, element_t P,
     element_ptr Qx, element_ptr Qy) {
-  int m;
+  mp_bitcnt_t m;
   element_t v;
   element_t Z;
   element_t a, b, c;
@@ -272,7 +272,8 @@ static void cc_miller_no_denom_proj(element_t res, mpz_t q, element_t P,
   Zy = curve_x_coord(Z);
 
   element_set1(v);
-  m = mpz_sizeinbase(q, 2) - 2;
+  m = (mp_bitcnt_t)mpz_sizeinbase(q, 2);
+  m = (m > 2 ? m - 2 : 0);
 
   for(;;) {
     do_tangent();
@@ -306,7 +307,7 @@ static void cc_miller_no_denom_proj(element_t res, mpz_t q, element_t P,
 
 static void cc_miller_no_denom_affine(element_t res, mpz_t q, element_t P,
     element_ptr Qx, element_ptr Qy) {
-  int m;
+  mp_bitcnt_t m;
   element_t v;
   element_t Z;
   element_t a, b, c;
@@ -378,7 +379,8 @@ static void cc_miller_no_denom_affine(element_t res, mpz_t q, element_t P,
   Zy = curve_y_coord(Z);
 
   element_set1(v);
-  m = mpz_sizeinbase(q, 2) - 2;
+  m = (mp_bitcnt_t)mpz_sizeinbase(q, 2);
+  m = (m > 2 ? m - 2 : 0);
 
   for(;;) {
     do_tangent();
@@ -417,7 +419,7 @@ static void lucas_even(element_ptr out, element_ptr in, mpz_t cofactor) {
   element_ptr v1 = element_y(out);
   element_ptr t0 = element_x(temp);
   element_ptr t1 = element_y(temp);
-  int j;
+  size_t j;
 
   element_set_si(t0, 2);
   element_double(t1, in0);
@@ -619,7 +621,7 @@ static void g_pairing_pp_init(pairing_pp_t p, element_ptr in1, pairing_t pairing
   const element_ptr Px = curve_x_coord(P);
   const element_ptr Py = curve_y_coord(P);
   element_t Z;
-  int m;
+  mp_bitcnt_t m;
   mnt_pairing_data_ptr info = pairing->data;
   element_t t0;
   element_t a, b, c;
@@ -692,7 +694,8 @@ static void g_pairing_pp_init(pairing_pp_t p, element_ptr in1, pairing_t pairing
   element_init(b, Fq);
   element_init(c, Fq);
 
-  m = mpz_sizeinbase(q, 2) - 2;
+  m = (mp_bitcnt_t)mpz_sizeinbase(q, 2);
+  m = (m > 2 ? m - 2 : 0);
   p->data = pbc_malloc(sizeof(pp_coeff_t) * 2 * m);
   coeff = (pp_coeff_t *) p->data;
   pp = coeff[0];
@@ -721,8 +724,9 @@ static void g_pairing_pp_init(pairing_pp_t p, element_ptr in1, pairing_t pairing
 static void g_pairing_pp_clear(pairing_pp_t p) {
   //TODO: better to store a sentinel value in p->data?
   mpz_ptr q = p->pairing->r;
-  int m = mpz_sizeinbase(q, 2) + mpz_popcount(q) - 3;
-  int i;
+  mp_bitcnt_t m = (mp_bitcnt_t)mpz_sizeinbase(q, 2) + mpz_popcount(q);
+  m = (m > 3 ? m - 3 : 0);
+  mp_bitcnt_t i;
   pp_coeff_t *coeff = (pp_coeff_t *) p->data;
   pp_coeff_ptr pp;
   for (i=0; i<m; i++) {
@@ -737,7 +741,8 @@ static void g_pairing_pp_clear(pairing_pp_t p) {
 static void g_pairing_pp_apply(element_ptr out, element_ptr in2, pairing_pp_t p) {
   mpz_ptr q = p->pairing->r;
   mnt_pairing_data_ptr info = p->pairing->data;
-  int m = mpz_sizeinbase(q, 2) - 2;
+  mp_bitcnt_t m = (mp_bitcnt_t)mpz_sizeinbase(q, 2);
+  m = (m > 2 ? m - 2 : 0);
   pp_coeff_t *coeff = (pp_coeff_t *) p->data;
   pp_coeff_ptr pp = coeff[0];
   element_ptr Qbase = in2;
@@ -937,7 +942,8 @@ static void g_pairing_ellnet(element_ptr out, element_ptr in1, element_ptr in2,
   element_init_same_as(u, d0);
   element_init_same_as(v, d0);
 
-  int m = mpz_sizeinbase(pairing->r, 2) - 2;
+  mp_bitcnt_t m = (mp_bitcnt_t)mpz_sizeinbase(pairing->r, 2);
+  m = (m > 2 ? m - 2 : 0);
   for (;;) {
     element_square(sm2, cm2);
     element_square(sm1, cm1);
@@ -1174,11 +1180,11 @@ static void compute_cm_curve(g_param_ptr param, pbc_cm_ptr cm) {
   element_init(hp, fpx);
 
   mpz_t *coefflist;
-  int n = pbc_hilbert(&coefflist, cm->D);
+  int n = (int)pbc_hilbert(&coefflist, cm->D);
 
   // Temporarily set the coefficient of x^{n-1} to 1 so hp has degree n - 1,
   // allowing us to use element_item().
-  poly_set_coeff1(hp, n - 1);
+  poly_set_coeff1(hp, (int)(n - 1));
   int i;
   for (i = 0; i < n; i++) {
     element_set_mpz(element_item(hp, i), coefflist[i]);
