@@ -14,8 +14,9 @@ void param_out_mpz(FILE *stream, char *s, mpz_t z);
 void param_out_int(FILE *stream, char *s, int i);
 // TODO: Replace with a stdarg function, e.g.
 //   err = lookup("ZZi", "p", "n", "l",  p->p, p->n, &p->l);
-int lookup_int(int *n, const char *(*tab)(const char *), const char *key);
-int lookup_mpz(mpz_t z, const char *(*tab)(const char *), const char *key);
+struct symtab_s; // let "include/pbc.h" not include "misc/symtab.h"
+int lookup_int(int *n, struct symtab_s *tab, const char *key);
+int lookup_mpz(mpz_t z, struct symtab_s *tab, const char *key);
 
 #pragma GCC visibility pop
 
