@@ -1,5 +1,4 @@
 #include <stdint.h> // for intptr_t
-#include <sys/time.h>
 #include "pbc_utils.h"
 
 typedef struct _FILETIME {
@@ -7,6 +6,11 @@ typedef struct _FILETIME {
   unsigned long dwHighDateTime;
 } FILETIME;
 
+typedef struct timeval {
+    long tv_sec;
+    long tv_usec;
+} timeval;
+
 void __stdcall GetSystemTimeAsFileTime(FILETIME*);
 
 int __cdecl gettimeofday(struct timeval* p, void* tz) {
