@@ -3,7 +3,7 @@
 // TODO: Garbage collection.
 // TODO: Recursion (stack frames), anonymous functions.
 
-#include <unistd.h>  // For getopt.
+#include <string.h>
 
 #include "pbc.h"
 #include "pbc_fp.h"
@@ -19,13 +19,15 @@
 #include "lex.yy.h"
 #include "parser.tab.h"
 
+// forward declaration to avoid C99 warnings
+int yyparse(void);
+
 int option_easy = 0;
 const char *option_prompt;
 
 char *pbc_getline(const char *prompt);
 
 void yyerror(char *s) { fprintf(stderr, "%s\n", s); }
-int yyparse(void);
 
 // Symbol table holding built-in functions and variables.
 static symtab_t reserved;
@@ -134,7 +136,7 @@ static val_ptr v_builtin(val_ptr v, tree_ptr t) {
   if (1 + n != darray_count(t->child)) {
     return val_new_error("%s: wrong number of arguments", fun->name);
   }
-  val_ptr arg[n];
+  val_ptr* arg = pbc_malloc(n * sizeof(val_ptr));
   int i;
   for(i = 0; i < n; i++) {
     arg[i] = tree_eval(darray_at(t->child, i));
@@ -142,7 +144,9 @@ static val_ptr v_builtin(val_ptr v, tree_ptr t) {
       return val_new_error("%s: argument %d type mismatch", fun->name, i + 1);
     }
   }
-  return fun->run(arg);
+  val_ptr out = fun->run(arg);
+  pbc_free(arg);
+  return out;
 }
 
 static void eval_stmt(void *ptr) {
@@ -443,6 +447,7 @@ tree_ptr tree_new_z(const char* s) {
 static val_ptr eval_err(tree_ptr t) {
   UNUSED_VAR(t);
   pbc_die("BUG: shouldn't reach here!");
+  return NULL;
 }
 
 tree_ptr tree_new_empty_stmt_list() {
@@ -572,7 +577,7 @@ static val_ptr run_item(val_ptr v[]) {
   mpz_t z;
   mpz_init(z);
   element_to_mpz(z, v[1]->elem);
-  int i = mpz_get_si(z);
+  int i = (int)mpz_get_si(z);
   mpz_clear(z);
   element_ptr a = element_item(v[0]->elem, i);
   element_ptr e = pbc_malloc(sizeof(*e));
@@ -716,7 +721,7 @@ static val_ptr run_exit(val_ptr v[]) {
   mpz_t z;
   mpz_init(z);
   element_to_mpz(z, v[0]->elem);
-  exit(mpz_get_si(z));
+  exit((int)mpz_get_si(z));
 }
 static fun_t fun_exit = {{ "exit", run_exit, 1, sig_elem }};
 
@@ -872,7 +877,7 @@ int yywrap_readline(void) {
     end_of_input = 1;
     return 1;
   }
-  int n = strlen(currentline);
+  size_t n = strlen(currentline);
   currentline = realloc(currentline, n + 2);
   currentline[n] = '\n';
   currentline[n + 1] = '\0';
